#ifdef __linux__
#include <stdio.h>
#else
#include <stdio.h>
#include <conio.h>
#include <Windows.h>
#endif

#include "libsangoma.h"
#include "libsangoma_wp_config.h"
#include "libsangoma_wp_config_priv.h"

#define WP_MAX_CARDS (16)
#define WP_MAX_CARD_PORTS (16)
#define WP_MAX_PORTS  (WP_MAX_CARDS*WP_MAX_CARD_PORTS)

#define SNG_OBJ_CHECK(x)  { if (!x) { DBG_ERR("%s: Error sangoma obj NUL \n", __FUNCTION__); return -1; } }
#define SNG_OBJ_CHECK_RET(x,ret)  { if (!x) { DBG_ERR("%s: Error sangoma obj NUL \n", __FUNCTION__); return ret; } }
#define SNG_OBJ_CHECK_VOID(x)  { if (!x) { DBG_ERR("%s: Error sangoma obj NUL \n", __FUNCTION__); return; } }

typedef struct {
	unsigned int wp_number;
	hardware_info_t wp_hw_info;
	wanpipe_port_cfg_t wp_hw_cfg;
} sangoma_hw_port_t;

typedef struct {
	sangoma_hw_port_t ports[WP_MAX_PORTS + 1];
	unsigned int number_of_ports;
} sangoma_hw_ports_t;

int libsng_dbg_level = 1;

/**************************************
Global variables
***************************************/
static sangoma_hw_ports_t sangoma_hw_ports;
static sangoma_hw_config_reg_t sangoma_hw_config_reg;
static int sangoma_hw_config_reg_init = 0;

/**************************************
Function definitions 
***************************************/

/**************************************
Function bodies
*continue**************************************/

#if 0
static void print_all_ports()
{
	int i;
	for (i = 1; i <= sangoma_hw_ports.number_of_ports; i++) {
		printf
			("Wanpipe%d  \t   card_model [%d]   firmware_version [%d]  pci_bus_number[%d]  pci_slot_number[%d]  port_number[%d] \n",
			 sangoma_hw_ports.ports[i].wp_number, sangoma_hw_ports.ports[i].wp_hw_info.card_model,
			 sangoma_hw_ports.ports[i].wp_hw_info.firmware_version, sangoma_hw_ports.ports[i].wp_hw_info.pci_bus_number,
			 sangoma_hw_ports.ports[i].wp_hw_info.pci_slot_number, sangoma_hw_ports.ports[i].wp_hw_info.port_number);
	}
}
#endif

static void sangoam_hw_ports_init()
{
	memset(&sangoma_hw_ports, 0, sizeof(sangoma_hw_ports));
}

static int sangoma_hw_config_init_check()
{
	if (!sangoma_hw_config_reg_init) {
		DBG_ERR("Error: libsangoma configuration libarary has not beein initialized \n");
		return -1;
	}

	return 0;
}

#if 0
static int sangoma_hw_init_port_cfg(sng_fd_t handle, sangoma_hw_port_t * sng_port)
{
	wanpipe_port_cfg_t port_cfg;
	int err;

	err = sangoma_driver_port_get_config(handle, &port_cfg, sng_port->wp_number);
	if (err) {
		return err;
	}

	memcpy(&sng_port->wp_hw_cfg, &port_cfg, sizeof(wanpipe_port_cfg_t));

	return 0;
}
#endif

static int sangoma_set_default_gsm_config(sangoma_hw_port_t * sng_port)
{
	wanpipe_port_cfg_t *cfg = &sng_port->wp_hw_cfg;
	wandev_conf_t *dev_cfg = &cfg->wandev_conf;
	sdla_fe_cfg_t *sdla_fe_cfg = &dev_cfg->fe_cfg;
	wanif_conf_t *if_cfg = &cfg->if_cfg[0];

	dev_cfg->magic = ROUTER_MAGIC;
	dev_cfg->config_id = WANCONFIG_AFT_GSM;
	dev_cfg->card_type = WANOPT_AFT_GSM;
	dev_cfg->S514_CPU_no[0] = SDLA_CPU_A;
	dev_cfg->comm_port = WANOPT_PRI;

	dev_cfg->PCI_slot_no = sng_port->wp_hw_info.pci_slot_number;
	dev_cfg->pci_bus_no = sng_port->wp_hw_info.pci_bus_number;

	dev_cfg->fe_cfg.cfg.te_cfg.active_ch = 0xFFFFFFFF;

	dev_cfg->mtu = 1500;
	dev_cfg->udp_port = 9000;
	dev_cfg->tdmv_conf.span_no = (unsigned char)sng_port->wp_number;

	sdla_fe_cfg->media = WAN_MEDIA_GSM;
	sdla_fe_cfg->line_no = sng_port->wp_number;
	sdla_fe_cfg->tdmv_law = WAN_TDMV_MULAW;

	cfg->port_no = sng_port->wp_number;
	cfg->num_of_ifs = 1;

	sprintf(if_cfg->name, "w%dg1", sng_port->wp_number);
	sprintf(if_cfg->usedby, SDLA_DECODE_USEDBY_FIELD(TDM_CHAN_VOICE_API));
	if_cfg->magic = ROUTER_MAGIC;
	if_cfg->active_ch = ENABLE_ALL_CHANNELS;
	if_cfg->mtu = 16;
	if_cfg->u.aft.mtu = 16;

	//printf("Default GSM configuratin set for wanpipe%i\n", sng_port->wp_number);

	return 0;					/* 0 is success */
}

static int sangoma_set_default_pri_tap_config(sangoma_hw_port_t * sng_port)
{
	wanpipe_port_cfg_t *cfg = &sng_port->wp_hw_cfg;
	wandev_conf_t *dev_cfg = &cfg->wandev_conf;
	sdla_fe_cfg_t *sdla_fe_cfg = &dev_cfg->fe_cfg;
	wanif_conf_t *if_cfg = &cfg->if_cfg[0];

	dev_cfg->magic = ROUTER_MAGIC;
	dev_cfg->config_id = WANCONFIG_AFT_TE1;
	dev_cfg->card_type = WANOPT_AFT;

	dev_cfg->S514_CPU_no[0] = SDLA_CPU_A;
	dev_cfg->comm_port = WANOPT_PRI;
	dev_cfg->PCI_slot_no = sng_port->wp_hw_info.pci_slot_number;
	dev_cfg->pci_bus_no = sng_port->wp_hw_info.pci_bus_number;

	dev_cfg->fe_cfg.cfg.te_cfg.active_ch = 0xFFFFFFFF;

	dev_cfg->mtu = 2048;
	dev_cfg->udp_port = 9000;
	dev_cfg->tdmv_conf.dchan = (1 << 15);	/* Channel 16. This is a bitmap, not a channel number. */
	dev_cfg->tdmv_conf.span_no = (unsigned char)sng_port->wp_number;
	dev_cfg->u.aft.hw_port_map = 1;

	sdla_fe_cfg->media = WAN_MEDIA_E1;
	sdla_fe_cfg->lcode = WAN_LCODE_HDB3;
	sdla_fe_cfg->frame = WAN_FR_CRC4;
	FE_LINENO(sdla_fe_cfg) = sng_port->wp_hw_info.port_number;

	FE_TDMV_LAW(sdla_fe_cfg) = WAN_TDMV_ALAW;

	FE_CLK(sdla_fe_cfg) = WAN_NORMAL_CLK;
	FE_REFCLK(sdla_fe_cfg) = 0;

	dev_cfg->hwec_conf.clk_src = WANOPT_NO;
	FE_NETWORK_SYNC(sdla_fe_cfg) = WANOPT_NO;

	sdla_fe_cfg->cfg.te_cfg.sig_mode = WAN_TE1_SIG_CCS;

#if 1
	sdla_fe_cfg->cfg.te_cfg.high_impedance_mode = 1;
	sdla_fe_cfg->cfg.te_cfg.rx_slevel = WAN_TE1_RX_SLEVEL_12_DB;
#else
	sdla_fe_cfg->cfg.te_cfg.high_impedance_mode = 0;
	sdla_fe_cfg->cfg.te_cfg.rx_slevel = WAN_TE1_RX_SLEVEL_43_DB;
#endif
	sdla_fe_cfg->cfg.te_cfg.lbo = WAN_E1_120;

	cfg->port_no = sng_port->wp_number;
	cfg->num_of_ifs = 1;

	sprintf(if_cfg->name, "w%dg1", sng_port->wp_number);
	sprintf(if_cfg->usedby, SDLA_DECODE_USEDBY_FIELD(TDM_CHAN_VOICE_API));

	if_cfg->magic = ROUTER_MAGIC;
	if_cfg->active_ch = ENABLE_ALL_CHANNELS;
	if_cfg->mtu = 80;
	if_cfg->u.aft.mtu = 80;
	if_cfg->u.aft.mru = 80;
	if_cfg->u.aft.mtu_idle = 80;
	if_cfg->hwec.enable = 1;

	dev_cfg->tdmv_conf.hw_dtmf = 0;

	return 0;					/* 0 is success */
}

static int sangoma_set_default_bri_tap_config(sangoma_hw_port_t * sng_port)
{
	wanpipe_port_cfg_t *cfg = &sng_port->wp_hw_cfg;
	wandev_conf_t *dev_cfg = &cfg->wandev_conf;
	sdla_fe_cfg_t *sdla_fe_cfg = &dev_cfg->fe_cfg;
	wanif_conf_t *if_cfg = &cfg->if_cfg[0];

	dev_cfg->magic = ROUTER_MAGIC;
	dev_cfg->config_id = WANCONFIG_AFT_ISDN_BRI;
	dev_cfg->card_type = WANOPT_AFT;

	dev_cfg->S514_CPU_no[0] = SDLA_CPU_A;
	dev_cfg->PCI_slot_no = sng_port->wp_hw_info.pci_slot_number;
	dev_cfg->pci_bus_no = sng_port->wp_hw_info.pci_bus_number;

	dev_cfg->fe_cfg.cfg.te_cfg.active_ch = 0xFFFFFFFF;

	dev_cfg->mtu = 1500;
	dev_cfg->udp_port = 9000;
	dev_cfg->tdmv_conf.span_no = (unsigned char)sng_port->wp_number;

	sdla_fe_cfg->media = WAN_MEDIA_BRI;
	FE_LINENO(sdla_fe_cfg) = sng_port->wp_hw_info.port_number;
	FE_TDMV_LAW(sdla_fe_cfg) = WAN_TDMV_ALAW;

	BRI_FE_CLK((*sdla_fe_cfg)) = WAN_NORMAL_CLK;
	FE_REFCLK(sdla_fe_cfg) = 0;

#if 1
	sdla_fe_cfg->cfg.te_cfg.high_impedance_mode = 1;
	sdla_fe_cfg->cfg.te_cfg.rx_slevel = WAN_TE1_RX_SLEVEL_12_DB;
#else
	sdla_fe_cfg->cfg.te_cfg.high_impedance_mode = 0;
	sdla_fe_cfg->cfg.te_cfg.rx_slevel = WAN_TE1_RX_SLEVEL_43_DB;
#endif
	cfg->port_no = sng_port->wp_number;
	cfg->num_of_ifs = 1;

	sprintf(if_cfg->name, "w%dg1", sng_port->wp_number);
	sprintf(if_cfg->usedby, SDLA_DECODE_USEDBY_FIELD(TDM_CHAN_VOICE_API));

	if_cfg->magic = ROUTER_MAGIC;
	if_cfg->active_ch = ENABLE_ALL_CHANNELS;
	if_cfg->mtu = 80;
	if_cfg->u.aft.mtu = 80;
	if_cfg->u.aft.mru = 80;
	if_cfg->u.aft.mtu_idle = 80;
	if_cfg->hwec.enable = 1;

	dev_cfg->tdmv_conf.hw_dtmf = 0;

	return 0;					/* 0 is success */
}

static int sangoma_set_default_isup_sig_tap_config(sangoma_hw_port_t * sng_port)
{
	wanpipe_port_cfg_t *cfg = &sng_port->wp_hw_cfg;
	wandev_conf_t *dev_cfg = &cfg->wandev_conf;
	sdla_fe_cfg_t *sdla_fe_cfg = &dev_cfg->fe_cfg;
	wanif_conf_t *if_cfg = &cfg->if_cfg[0];

	dev_cfg->magic = ROUTER_MAGIC;
	dev_cfg->config_id = WANCONFIG_AFT_TE1;
	dev_cfg->card_type = WANOPT_AFT;

	dev_cfg->S514_CPU_no[0] = SDLA_CPU_A;
	dev_cfg->comm_port = WANOPT_PRI;
	dev_cfg->PCI_slot_no = sng_port->wp_hw_info.pci_slot_number;
	dev_cfg->pci_bus_no = sng_port->wp_hw_info.pci_bus_number;

	dev_cfg->fe_cfg.cfg.te_cfg.active_ch = 0xFFFFFFFF;

	dev_cfg->mtu = 1500;
	dev_cfg->udp_port = 9000;
	dev_cfg->tdmv_conf.dchan = (1 << 15);	/* Channel 16. This is a bitmap, not a channel number. */
	dev_cfg->tdmv_conf.span_no = (unsigned char)sng_port->wp_number;
	/* flag to enable linear mapping of ports
	 * like A108 card physical port 1 will be mapped to wanpipe 1 and 2 port and so on..
	 * */
	dev_cfg->u.aft.hw_port_map = 1;

	sdla_fe_cfg->media = WAN_MEDIA_E1;
	sdla_fe_cfg->lcode = WAN_LCODE_HDB3;
	sdla_fe_cfg->frame = WAN_FR_NCRC4;
	FE_LINENO(sdla_fe_cfg) = sng_port->wp_hw_info.port_number;

	FE_TDMV_LAW(sdla_fe_cfg) = WAN_TDMV_ALAW;

	FE_CLK(sdla_fe_cfg) = WAN_NORMAL_CLK;
	FE_REFCLK(sdla_fe_cfg) = 0;

	dev_cfg->hwec_conf.clk_src = WANOPT_NO;
	FE_NETWORK_SYNC(sdla_fe_cfg) = WANOPT_NO;

	sdla_fe_cfg->cfg.te_cfg.sig_mode = WAN_TE1_SIG_CCS;

#if 1
	sdla_fe_cfg->cfg.te_cfg.high_impedance_mode = 1;
	sdla_fe_cfg->cfg.te_cfg.rx_slevel = WAN_TE1_RX_SLEVEL_12_DB;
#else
	sdla_fe_cfg->cfg.te_cfg.high_impedance_mode = 0;
	sdla_fe_cfg->cfg.te_cfg.rx_slevel = WAN_TE1_RX_SLEVEL_43_DB;
#endif
	sdla_fe_cfg->cfg.te_cfg.lbo = WAN_E1_120;

	cfg->port_no = sng_port->wp_number;
	cfg->num_of_ifs = 1;

	sprintf(if_cfg->name, "w%dg1", sng_port->wp_number);
	sprintf(if_cfg->usedby, SDLA_DECODE_USEDBY_FIELD(TDM_CHAN_VOICE_API));

	if_cfg->magic = ROUTER_MAGIC;
	if_cfg->active_ch = ENABLE_ALL_CHANNELS;
	if_cfg->mtu = 80;
	if_cfg->u.aft.mtu = 80;
	if_cfg->u.aft.mru = 80;
	if_cfg->u.aft.mtu_idle = 80;
	if_cfg->hwec.enable = 1;

	/* Below fields has to be enable for wanpipe to work in SS7 mode */
	if_cfg->u.aft.mtp1_filter = 1;
	if_cfg->u.aft.sw_hdlc = 1;

	dev_cfg->tdmv_conf.hw_dtmf = 0;

	return 0;					/* 0 is success */
}

int _LIBSNG_CALL sangoma_set_default_isup_pure_voice_span_config(sangoma_port_obj_t sangoma_port_obj)
{
	sangoma_hw_port_t *sng_port = (sangoma_hw_port_t *) sangoma_port_obj;

	wanpipe_port_cfg_t *cfg = &sng_port->wp_hw_cfg;
	wandev_conf_t *dev_cfg = &cfg->wandev_conf;
	sdla_fe_cfg_t *sdla_fe_cfg = &dev_cfg->fe_cfg;
	wanif_conf_t *if_cfg = &cfg->if_cfg[0];

	dev_cfg->magic = ROUTER_MAGIC;
	dev_cfg->config_id = WANCONFIG_AFT_TE1;
	dev_cfg->card_type = WANOPT_AFT;

	dev_cfg->S514_CPU_no[0] = SDLA_CPU_A;
	dev_cfg->comm_port = WANOPT_PRI;
	dev_cfg->PCI_slot_no = sng_port->wp_hw_info.pci_slot_number;
	dev_cfg->pci_bus_no = sng_port->wp_hw_info.pci_bus_number;

	dev_cfg->fe_cfg.cfg.te_cfg.active_ch = 0xFFFFFFFF;

	dev_cfg->mtu = 1500;
	dev_cfg->udp_port = 9000;
	dev_cfg->tdmv_conf.span_no = (unsigned char)sng_port->wp_number;
	/* flag to enable linear mapping of ports
	 * like A108 card physical port 1 will be mapped to wanpipe 1 and 2 port and so on..
	 * */
	dev_cfg->u.aft.hw_port_map = 1;

	sdla_fe_cfg->media = WAN_MEDIA_E1;
	sdla_fe_cfg->lcode = WAN_LCODE_HDB3;
	sdla_fe_cfg->frame = WAN_FR_NCRC4;
	FE_LINENO(sdla_fe_cfg) = sng_port->wp_hw_info.port_number;

	FE_TDMV_LAW(sdla_fe_cfg) = WAN_TDMV_ALAW;

	FE_CLK(sdla_fe_cfg) = WAN_NORMAL_CLK;
	FE_REFCLK(sdla_fe_cfg) = 0;

	dev_cfg->hwec_conf.clk_src = WANOPT_NO;
	FE_NETWORK_SYNC(sdla_fe_cfg) = WANOPT_NO;

	sdla_fe_cfg->cfg.te_cfg.sig_mode = WAN_TE1_SIG_CCS;

#if 1
	sdla_fe_cfg->cfg.te_cfg.high_impedance_mode = 1;
	sdla_fe_cfg->cfg.te_cfg.rx_slevel = WAN_TE1_RX_SLEVEL_12_DB;
#else
	sdla_fe_cfg->cfg.te_cfg.high_impedance_mode = 0;
	sdla_fe_cfg->cfg.te_cfg.rx_slevel = WAN_TE1_RX_SLEVEL_43_DB;
#endif
	sdla_fe_cfg->cfg.te_cfg.lbo = WAN_E1_120;

	cfg->port_no = sng_port->wp_number;
	cfg->num_of_ifs = 1;

	sprintf(if_cfg->name, "w%dg1", sng_port->wp_number);
	sprintf(if_cfg->usedby, SDLA_DECODE_USEDBY_FIELD(TDM_CHAN_VOICE_API));

	if_cfg->magic = ROUTER_MAGIC;
	if_cfg->active_ch = ENABLE_ALL_CHANNELS;
	if_cfg->mtu = 80;
	if_cfg->u.aft.mtu = 80;
	if_cfg->u.aft.mru = 80;
	if_cfg->u.aft.mtu_idle = 80;
	if_cfg->hwec.enable = 1;

	dev_cfg->tdmv_conf.hw_dtmf = 0;

	return 0;					/* 0 is success */
}

static int sangoma_set_default_a200_config(sangoma_hw_port_t * sng_port)
{
	wanpipe_port_cfg_t *cfg = &sng_port->wp_hw_cfg;
	wandev_conf_t *dev_cfg = &cfg->wandev_conf;
	sdla_fe_cfg_t *sdla_fe_cfg = &dev_cfg->fe_cfg;
	wanif_conf_t *if_cfg = &cfg->if_cfg[0];

	dev_cfg->magic = ROUTER_MAGIC;
	dev_cfg->config_id = WANCONFIG_AFT_ANALOG;
	dev_cfg->card_type = WANOPT_AFT;
	dev_cfg->S514_CPU_no[0] = SDLA_CPU_A;
	dev_cfg->comm_port = WANOPT_PRI;

	dev_cfg->PCI_slot_no = sng_port->wp_hw_info.pci_slot_number;
	dev_cfg->pci_bus_no = sng_port->wp_hw_info.pci_bus_number;

	dev_cfg->mtu = 1500;
	dev_cfg->udp_port = 9000;
	dev_cfg->tdmv_conf.span_no = (unsigned char)sng_port->wp_number;

	sdla_fe_cfg->media = WAN_MEDIA_FXOFXS;
	sdla_fe_cfg->line_no = sng_port->wp_number;
	sdla_fe_cfg->tdmv_law = WAN_TDMV_MULAW;

	sdla_fe_cfg->cfg.remora.battthresh = 3;
	sdla_fe_cfg->cfg.remora.battdebounce = 16;
	sdla_fe_cfg->cfg.remora.ohthresh = 10;			/* DEFAULT_OH_THRESH */
	sdla_fe_cfg->cfg.remora.rm_mode = WAN_RM_TAPPING;	/* DEFAULT_OH_THRESH */

	cfg->port_no = sng_port->wp_number;
	cfg->num_of_ifs = 1;

	sprintf(if_cfg->name, "w%dg1", sng_port->wp_number);
	sprintf(if_cfg->usedby, SDLA_DECODE_USEDBY_FIELD(TDM_CHAN_VOICE_API));
	if_cfg->magic = ROUTER_MAGIC;
	if_cfg->active_ch = ENABLE_ALL_CHANNELS;
	if_cfg->mtu = 8;
	if_cfg->u.aft.mtu = 8;

	return 0;					/* 0 is success */
}

static int sangoma_set_default_mtp2_api_config(sangoma_hw_port_t * sng_port, int operating_mode)
{
	wanpipe_port_cfg_t *cfg = &sng_port->wp_hw_cfg;
	wandev_conf_t *dev_cfg = &cfg->wandev_conf;
	sdla_fe_cfg_t *sdla_fe_cfg = &dev_cfg->fe_cfg;
	wanif_conf_t *if_cfg = &cfg->if_cfg[0];

	dev_cfg->magic = ROUTER_MAGIC;
	dev_cfg->config_id = WANCONFIG_AFT_TE1;
	dev_cfg->card_type = WANOPT_AFT;

	dev_cfg->S514_CPU_no[0] = SDLA_CPU_A;
	dev_cfg->comm_port = WANOPT_PRI;
	dev_cfg->PCI_slot_no = sng_port->wp_hw_info.pci_slot_number;
	dev_cfg->pci_bus_no = sng_port->wp_hw_info.pci_bus_number;

	dev_cfg->fe_cfg.cfg.te_cfg.active_ch = 0xFFFFFFFF;

	dev_cfg->mtu = 1500;
	dev_cfg->udp_port = 9000;

	if (operating_mode == SNG_SS7_MTP2_API_MODE_LSL) {
		/* chan number must be 1-31 */
		dev_cfg->tdmv_conf.dchan = ((1 << 30) | (1 << 29) | (1 << 28)| (1 << 27) | (1 << 26) | (1 << 25) | (1 << 24) | (1 << 23) | (1 << 22) | (1 << 21) | (1 << 20) | (1 << 19) | (1 << 18) | (1 << 17) | (1 << 16) | (1 << 15) | (1 << 14) | (1 << 13) | (1 << 12) | (1 << 11) | (1 << 10) | (1 << 9) | (1 << 8) | (1 << 7) | (1 << 6) | (1 << 5) | (1 << 4) |(1 << 3) | (1 << 2) | (1 << 1) | 1);
	} else {
		/* chan number must be 1 */
		dev_cfg->tdmv_conf.dchan = 1;
	}

	dev_cfg->tdmv_conf.span_no = (unsigned char)sng_port->wp_number;
	dev_cfg->ttl = 255;

	sdla_fe_cfg->media = WAN_MEDIA_E1;
	sdla_fe_cfg->lcode = WAN_LCODE_HDB3;
	sdla_fe_cfg->frame = WAN_FR_CRC4;
	sdla_fe_cfg->cfg.te_cfg.ais_maintenance = 0;
	FE_LINENO(sdla_fe_cfg) = sng_port->wp_hw_info.port_number;

	FE_TDMV_LAW(sdla_fe_cfg) = WAN_TDMV_ALAW;

	FE_CLK(sdla_fe_cfg) = WAN_NORMAL_CLK;       /*  WAN_MASTER_CLK master when you are running both mtp2 on a single machine and mtp2 api app */
	FE_REFCLK(sdla_fe_cfg) = 0;

	dev_cfg->hwec_conf.clk_src = WANOPT_NO;
	FE_NETWORK_SYNC(sdla_fe_cfg) = WANOPT_NO;

	sdla_fe_cfg->cfg.te_cfg.sig_mode = WAN_TE1_SIG_CCS;
	sdla_fe_cfg->cfg.te_cfg.high_impedance_mode = 0;
	sdla_fe_cfg->cfg.te_cfg.rx_slevel = WAN_TE1_RX_SLEVEL_43_DB;
	sdla_fe_cfg->cfg.te_cfg.lbo = WAN_E1_120;

	cfg->port_no = sng_port->wp_number;
	cfg->num_of_ifs = 1;

	sprintf(if_cfg->name, "w%dg1", sng_port->wp_number);
	sprintf(if_cfg->usedby, SDLA_DECODE_USEDBY_FIELD(TDM_CHAN_VOICE_API));

	if_cfg->magic = ROUTER_MAGIC;
	if_cfg->active_ch = ENABLE_ALL_CHANNELS;
	if_cfg->mtu = 80;
	if_cfg->u.aft.mtu = 80;
	if_cfg->u.aft.mru = 80;
	if_cfg->u.aft.mtu_idle = 80;
	if_cfg->hwec.enable = 0;

	/* Below fields has to be enable for wanpipe to work in mtp2 api mode */
	if_cfg->hdlc_streaming = 1;
	if_cfg->u.aft.hdlc_repeat = 1;
	
	if (operating_mode == SNG_SS7_MTP2_API_MODE_LSL) {
		/* LSL configuration */
		if_cfg->u.aft.ss7_mode = 0;
		if_cfg->u.aft.ss7_lssu_size = 4; /* 6 or 7 for HSL mode and 4 or 5 for LSL mode */
	} else {
		/* HSL configuration */
		if_cfg->u.aft.ss7_mode = 1;
		if_cfg->u.aft.ss7_lssu_size = 6; /* 6 or 7 for HSL mode and 4 or 5 for LSL mode */
	}

	return 0;                                       /* 0 is success */
}

/* ============================================================= 
   Public Funcitons 

*/

int _LIBSNG_CALL sangoma_set_wp_config(sangoma_port_obj_t sangoma_port_obj, sng_wp_config_t* app_cfg)
{
	sangoma_hw_port_t *sng_port = (sangoma_hw_port_t *) sangoma_port_obj;
	wanpipe_port_cfg_t *cfg = &sng_port->wp_hw_cfg;
	wandev_conf_t *dev_cfg = &cfg->wandev_conf;
	sdla_fe_cfg_t *sdla_fe_cfg = &dev_cfg->fe_cfg;
	/* can be used in future */
/*	wanif_conf_t *if_cfg = &cfg->if_cfg[0];*/

	sdla_fe_cfg->frame = app_cfg->frame_mode;
	sdla_fe_cfg->lcode = app_cfg->line_code;
	sdla_fe_cfg->media = app_cfg->media;
	sdla_fe_cfg->cfg.te_cfg.high_impedance_mode = app_cfg->high_impedance_mode;
	if (sdla_fe_cfg->cfg.te_cfg.high_impedance_mode) {
		sdla_fe_cfg->cfg.te_cfg.rx_slevel = WAN_TE1_RX_SLEVEL_12_DB;
	} else {
		sdla_fe_cfg->cfg.te_cfg.rx_slevel = WAN_TE1_RX_SLEVEL_43_DB; 
	}

	return 0;
}

int _LIBSNG_CALL sangoma_config_reg(sangoma_hw_config_reg_t * reg)
{
	SNG_OBJ_CHECK(reg);
	memcpy(&sangoma_hw_config_reg, reg, sizeof(sangoma_hw_config_reg_t));
	sangoma_hw_config_reg_init = 1;
	return 0;
}

int _LIBSNG_CALL sangoma_config_port_cfg_set_option(sangoma_port_obj_t sangoma_port_obj, char *key, char *value)
{

	return 0;
}

int _LIBSNG_CALL sangoma_config_port_cfg_get_option(sangoma_port_obj_t sangoma_port_obj, char *key, char *value)
{

	return 0;
}

int _LIBSNG_CALL sangoma_config_port_cfg_dump(sangoma_port_obj_t sangoma_port_obj)
{

	return 0;
}

unsigned int _LIBSNG_CALL sangoma_config_port_get_port_id(sangoma_port_obj_t sangoma_port_obj)
{
	sangoma_hw_port_t *sng_port = (sangoma_hw_port_t *) sangoma_port_obj;
	SNG_OBJ_CHECK(sng_port);
	return sng_port->wp_number;
}

int _LIBSNG_CALL sangoma_config_port_hwinfo_dump(sangoma_port_obj_t sangoma_port_obj)
{
	sangoma_hw_port_t *sng_port = (sangoma_hw_port_t *) sangoma_port_obj;
	SNG_OBJ_CHECK(sng_port);

	DBG_EVNT("card_model       : %s (%d)\n",
			 SDLA_ADPTR_NAME(sng_port->wp_hw_info.card_model), sng_port->wp_hw_info.card_model);
	DBG_EVNT("firmware_version\t: 0x%02X\n", sng_port->wp_hw_info.firmware_version);
	DBG_EVNT("pci_bus_number\t\t: %d\n", sng_port->wp_hw_info.pci_bus_number);
	DBG_EVNT("pci_slot_number\t\t: %d\n", sng_port->wp_hw_info.pci_slot_number);
	DBG_EVNT("max_hw_ec_chans\t\t: %d\n", sng_port->wp_hw_info.max_hw_ec_chans);
	DBG_EVNT("port_number\t\t: %d\n", sng_port->wp_hw_info.port_number);

	return 0;
}

char *sangoma_config_port_hwinfo_get_model_str(sangoma_port_obj_t sangoma_port_obj)
{
	sangoma_hw_port_t *sng_port = (sangoma_hw_port_t *) sangoma_port_obj;
	SNG_OBJ_CHECK_RET(sng_port, "NULL");
	return SDLA_ADPTR_NAME(sng_port->wp_hw_info.card_model);
}

unsigned int _LIBSNG_CALL sangoma_config_port_hwinfo_get_model_id(sangoma_port_obj_t sangoma_port_obj)
{
	sangoma_hw_port_t *sng_port = (sangoma_hw_port_t *) sangoma_port_obj;
	SNG_OBJ_CHECK(sng_port);
	return sng_port->wp_hw_info.card_model;
}

unsigned int _LIBSNG_CALL sangoma_config_port_hwinfo_get_pcislot(sangoma_port_obj_t sangoma_port_obj)
{
	sangoma_hw_port_t *sng_port = (sangoma_hw_port_t *) sangoma_port_obj;
	SNG_OBJ_CHECK(sng_port);
	return sng_port->wp_hw_info.pci_slot_number;
}

unsigned int _LIBSNG_CALL sangoma_config_port_hwinfo_get_pcibus(sangoma_port_obj_t sangoma_port_obj)
{
	sangoma_hw_port_t *sng_port = (sangoma_hw_port_t *) sangoma_port_obj;
	SNG_OBJ_CHECK(sng_port);
	return sng_port->wp_hw_info.pci_bus_number;
}

unsigned int _LIBSNG_CALL sangoma_config_port_hwinfo_get_port_number(sangoma_port_obj_t sangoma_port_obj)
{
	sangoma_hw_port_t *sng_port = (sangoma_hw_port_t *) sangoma_port_obj;
	SNG_OBJ_CHECK(sng_port);
	return sng_port->wp_hw_info.port_number;
}

unsigned int _LIBSNG_CALL sangoma_config_port_hwinfo_get_fimrware_version(sangoma_port_obj_t sangoma_port_obj)
{
	sangoma_hw_port_t *sng_port = (sangoma_hw_port_t *) sangoma_port_obj;
	SNG_OBJ_CHECK(sng_port);
	return sng_port->wp_hw_info.firmware_version;
}

int _LIBSNG_CALL sangoma_config_port_save(sangoma_port_obj_t sangoma_port_obj)
{
	sangoma_hw_port_t *sng_port = (sangoma_hw_port_t *) sangoma_port_obj;
	sng_fd_t h_port;
	SNG_OBJ_CHECK(sng_port);

	h_port = sangoma_open_driver_ctrl(sng_port->wp_number);
	if (h_port == INVALID_HANDLE_VALUE) {
		DBG_ERR("%s: Failed opening device wanpipe%d. \n", __FUNCTION__, sng_port->wp_number);
		return -1;
	}

	if (0 != sangoma_driver_port_set_config(h_port, &sng_port->wp_hw_cfg, sng_port->wp_number)) {
		DBG_ERR("Wanpipe%d configuration failed. \n");

		DBG_ERR("%s: Wanpipe%d configuration failed. \n", __FUNCTION__, sng_port->wp_number);
		sangoma_close(&h_port);
		return -1;
	}
	if (0 /*SAN_STATUS_SUCCESS */  != sng_port->wp_hw_cfg.operation_status) {
		DBG_ERR("%s: Wanpipe%d configuration failed. Cause : %d. \n", __FUNCTION__, sng_port->wp_number,
				sng_port->wp_hw_cfg.operation_status);
		sangoma_close(&h_port);
		return sng_port->wp_hw_cfg.operation_status;
	}

	sangoma_close(&h_port);

	return 0;					/* 0 is success */
}

int _LIBSNG_CALL sangoma_config_port_apply(sangoma_port_obj_t sangoma_port_obj)
{

	return 0;
}

int _LIBSNG_CALL sangoma_config_port_start(sangoma_port_obj_t sangoma_port_obj)
{
	int ret = 0;
	sangoma_hw_port_t *sng_port = (sangoma_hw_port_t *) sangoma_port_obj;
	port_management_struct_t port_mgmt;
	sng_fd_t h_port;

	SNG_OBJ_CHECK(sng_port);

	h_port = sangoma_open_driver_ctrl(sng_port->wp_number);
	if (h_port == INVALID_HANDLE_VALUE) {
		DBG_ERR("Failed opening device wanpipe%d. \n", sng_port->wp_number);
		return -1;
	}
	memset(&port_mgmt, 0, sizeof(port_mgmt));
	ret = sangoma_driver_port_start(h_port, &port_mgmt, sng_port->wp_number);
#ifdef ENABLE_EC
	if (ret == 0) {
		char devname[128];
#if defined(__WINDOWS__)
		/* must add 'if1' after 'wanpipe?' */
		wp_snprintf(devname, sizeof(devname), "wanpipe%d_if1", sng_port->wp_number);
#else
		wp_snprintf(devname, sizeof(devname), "wanpipe%d", sng_port->wp_number);
#endif
		ret = wanec_api_config(devname, 0, NULL);
		if (ret == 0) {
        		wanec_api_opmode_t      opmode;
        		memset(&opmode, 0, sizeof(wanec_api_opmode_t));
        		opmode.mode=WANEC_API_OPMODE_NORMAL;
        		opmode.fe_chan_map      = 0xFFFFFFFF;
        		ret=wanec_api_opmode(devname, 0, &opmode);
			if (ret == 0) {
				sangoma_hwec_enable(devname,opmode.fe_chan_map);	
			}
		}
	}
#endif

	sangoma_close(&h_port);
	return ret;					/* 0 is success */
}

int _LIBSNG_CALL sangoma_config_port_stop(sangoma_port_obj_t sangoma_port_obj)
{

	int ret = 0;
	sangoma_hw_port_t *sng_port = (sangoma_hw_port_t *) sangoma_port_obj;
	port_management_struct_t port_mgmt;
	sng_fd_t h_port;

	SNG_OBJ_CHECK(sng_port);

	h_port = sangoma_open_driver_ctrl(sng_port->wp_number);
	if (h_port == INVALID_HANDLE_VALUE) {
		DBG_ERR("Failed opening device wanpipe%d. \n", sng_port->wp_number);
		return -1;
	}
	memset(&port_mgmt, 0, sizeof(port_mgmt));
	ret = sangoma_driver_port_stop(h_port, &port_mgmt, sng_port->wp_number);

	sangoma_close(&h_port);
	return ret;					/* 0 is success */
}

int _LIBSNG_CALL sangoma_config_hwprobe(int *detected, int *configured, int operating_mode)
{
	unsigned int wp_index;
	port_management_struct_t port_mgmt;
	int result = 0;
	int i;
	int errs = 0;
	int err;
	sng_fd_t wp_handle = INVALID_HANDLE_VALUE;

	if (sangoma_hw_config_init_check()) {
		return -1;
	}

	if (!detected || !configured) {
		return -1;
	}

	*detected = 0;
	*configured = 0;

#ifdef __linux__
	system("modprobe wanpipe > /dev/null 2> /dev/null");
	system("modprobe wanec > /dev/null 2> /dev/null");
#else
	system("wanrouter load");
#endif

	libsng_dbg_level = 1;

	memset(&port_mgmt, 0, sizeof(port_mgmt));
	sangoam_hw_ports_init();

	/* Wait up to 5sec for a device to be creaed by udev (Linux) */
	for (i = 0; i < 5; i++) {
		wp_handle = sangoma_open_driver_ctrl(i);
		if (wp_handle == INVALID_HANDLE_VALUE) {
			sangoma_msleep(1000);
		} else {
			sangoma_close(&wp_handle);
			break;
		}
	}

	for (wp_index = 1; wp_index <= (WP_MAX_PORTS); wp_index++) {

		/* Already probed */
		if (sangoma_hw_ports.ports[wp_index].wp_number) {
			continue;
		}

		wp_handle = INVALID_HANDLE_VALUE;
		wp_handle = sangoma_open_driver_ctrl(wp_index);
		if (wp_handle == INVALID_HANDLE_VALUE) {
#ifdef __WINDOWS__
			/* on windows, the card index might not be started from 1 */
			continue;
#else
			if (wp_index == 1) {
				DBG_ERR("Error: Failed to open sangoma driver ctrl device\n");
				return -1;
			} else {
				sangoma_close(&wp_handle);
				break;
			}
#endif
		}

		result = sangoma_driver_get_hw_info(wp_handle, &port_mgmt, wp_index);
		if (result) {
			sangoma_close(&wp_handle);
			break;
		}

		memcpy(&sangoma_hw_ports.ports[wp_index].wp_hw_info, port_mgmt.data, sizeof(hardware_info_t));
		sangoma_hw_ports.ports[wp_index].wp_number = wp_index;

		switch (sangoma_hw_ports.ports[wp_index].wp_hw_info.card_model) {
		case A101_ADPTR_2TE1:
		case A104_ADPTR_4TE1:
		case A108_ADPTR_8TE1:
		case A116_ADPTR_16TE1:
			if (operating_mode == SNG_ISDN_PRI_TAPPING_MODE) {
				sangoma_set_default_pri_tap_config(&sangoma_hw_ports.ports[wp_index]);
			} else if (operating_mode == SNG_SS7_ISUP_SIG_TAPPING_MODE) {
				sangoma_set_default_isup_sig_tap_config(&sangoma_hw_ports.ports[wp_index]);
			} else if ((operating_mode == SNG_SS7_MTP2_API_MODE_LSL) || (operating_mode == SNG_SS7_MTP2_API_MODE_HSL)) {
				sangoma_set_default_mtp2_api_config(&sangoma_hw_ports.ports[wp_index], operating_mode);
			} else {
				sangoma_hw_ports.ports[wp_index].wp_number = 0;
				DBG_ERR("%s: Warning: Unsupported HW Type %s %d\n",
						__FUNCTION__,
						SDLA_ADPTR_NAME(sangoma_hw_ports.ports[wp_index].wp_hw_info.card_model),
						sangoma_hw_ports.ports[wp_index].wp_hw_info.card_model);
				sangoma_close(&wp_handle);
				continue;
			}
			break;

		case AFT_ADPTR_ISDN:
			sangoma_set_default_bri_tap_config(&sangoma_hw_ports.ports[wp_index]);
			break;

		case AFT_ADPTR_W400:
			sangoma_set_default_gsm_config(&sangoma_hw_ports.ports[wp_index]);
			break;

		case A200_ADPTR_ANALOG:
		case A400_ADPTR_ANALOG:
			sangoma_set_default_a200_config(&sangoma_hw_ports.ports[wp_index]);
			break;
				
		default:
			sangoma_hw_ports.ports[wp_index].wp_number = 0;
			DBG_ERR("%s: Warning: Unsupported HW Type %s %d\n",
					__FUNCTION__,
					SDLA_ADPTR_NAME(sangoma_hw_ports.ports[wp_index].wp_hw_info.card_model),
					sangoma_hw_ports.ports[wp_index].wp_hw_info.card_model);
			sangoma_close(&wp_handle);
			continue;
		}
		sangoma_close(&wp_handle);

		/* Stop all the port first, so that driver is in a known condition */
		sangoma_config_port_stop(&sangoma_hw_ports.ports[wp_index]);
		sangoma_hw_ports.number_of_ports++;
	}

	for (wp_index = 1; wp_index <= (WP_MAX_PORTS); wp_index++) {

		if (!sangoma_hw_ports.ports[wp_index].wp_number) {
			continue;
		}

		err = sangoma_hw_config_reg.hwprobe_callback((sangoma_port_obj_t) & sangoma_hw_ports.ports[wp_index],
													 sangoma_hw_ports.ports[wp_index].wp_number,
													 sangoma_hw_config_reg.user_obj);
		if (err < 0) {
			errs++;
		} else if (err == 0) {
			*configured = *configured + 1;
		} else {
			/* Port config skipped */
			sangoma_hw_ports.ports[wp_index].wp_number = 0;
		}
		*detected = *detected + 1;

		sangoma_hw_ports.number_of_ports++;
	}

	/* In case of tapping all the spans needs to be configured before they get started as in case of SS7/ISDN Tapping
	 * one has to configure a peer span for each span */
	for (wp_index = 1; wp_index <= (WP_MAX_PORTS); wp_index++) {

		if (!sangoma_hw_ports.ports[wp_index].wp_number) {
			continue;
		}

		if (sangoma_hw_config_reg.hwprobe_callback_post) {
			err = sangoma_hw_config_reg.hwprobe_callback_post((sangoma_port_obj_t) & sangoma_hw_ports.ports[wp_index],
															  sangoma_hw_ports.ports[wp_index].wp_number,
															  sangoma_hw_config_reg.user_obj);
			if (err < 0) {
				errs++;
			}
		}
	}

	return (errs ? -1 : 0);
}

int _LIBSNG_CALL sangoma_stop_all_wanpipe()
{
	int wp_index = 0;

	for (wp_index = 1; wp_index <= (WP_MAX_PORTS); wp_index++) {
		if (sangoma_hw_ports.ports[wp_index].wp_number != 0) {
			sangoma_config_port_stop(&sangoma_hw_ports.ports[wp_index]);
		}
	}

	return 0;
}
