/*
 * Copyright (c) 2014, 
 * Sangoma Technologies
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * * Redistributions of source code must retain the above copyright
 *   notice, this list of conditions and the following disclaimer.
 *
 * * Redistributions in binary form must reproduce the above copyright
 *   notice, this list of conditions and the following disclaimer in the
 *   documentation and/or other materials provided with the distribution.
 *
 * * Neither the name of the original author; nor the names of any contributors
 *   may be used to endorse or promote products derived from this software
 *   without specific prior written permission.
 *
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT OWNER
 * OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * Contributors: Nenad Corbic <ncorbic@sangoma.com>
 * 		 Pushkar Singh <psingh@sangoma.com>
 *
 */

#ifndef __RUN_MTP2_H__
#define __RUN_MTP2_H__

/********************************************************************************/
/*                                                                              */
/*                              Library Inclusion                               */
/*                                                                              */
/********************************************************************************/
#include <stdlib.h> /* required to generate random numbers */
#include <signal.h>
#include <stdarg.h>
#include <ctype.h>
#ifdef __WINDOWS__
#include <io.h>
#include <sys/stat.h>
#endif

#include <fcntl.h>
#include "libsangoma_freetdm.h"

#include "../../libs/freetdm/src/ftmod/ftmod_sangoma_ss7/ftmod_sangoma_ss7_mtp2api.h"
#include "private/ftdm_core.h"

/********************************************************************************/
/*                                                                              */
/*                                     MACROS                                   */
/*                                                                              */
/********************************************************************************/
#define NUM_E1_CHANS 31
#define MAX_MSU_SIZE 272

/* MTP2 API CARDTYPE DECODE **********************************************************/
#define SNG_MTP2_API_CARD_TYPE(card_type) \
((card_type) == A101_ADPTR_1TE1)	?"A101 single E1/T1 channels" : \
((card_type) == A101_ADPTR_2TE1)      	?"A101 bi E1/T1 channels" : \
((card_type) == A104_ADPTR_4TE1)        ?"A104 quad line E1/T1" : \
((card_type) == A108_ADPTR_8TE1)        ?"A108 octa E1/T1 channels" : \
((card_type) == A116_ADPTR_16TE1)	?"A116 16 channels E1/T1" : \
((card_type) == AFT_ADPTR_ISDN)		?"A 500 BRI card" : \
((card_type) == AFT_ADPTR_W400)         ?"W400 GSM" : \
"Unknown"

/********************************************************************************/
/*                                                                              */
/*                         Structure Declearation                               */
/*                                                                              */
/********************************************************************************/
typedef enum mtp2_link_flags {
	LINK_STATE_CONNECTED    = (1<<0),
	LINK_STATE_CONGESTED    = (1<<1),
} mtp2_link_flags_t;

typedef struct chan_data {
	uint8_t inuse;
	uint32_t span_id; /* Physical span number */
	uint32_t chan_id;       /* Physical chan number */
	mtp2_link_flags_t flags;
	unsigned long rx_frames;
	unsigned long tx_frames;
	uint8_t rx_seq;
	ftdm_channel_t *ftdmchan;
} chan_data_t;

typedef struct span_data {
	int init;
	int inuse;
	ftdm_span_t *ftdmspan;
	void *sangoma_port_obj;
	chan_data_t channels[NUM_E1_CHANS+500];
} span_data_t;

/*!
  \ in order to provide user flexibility to configure
  \FreeTDM SS7 span for SS7 modules

*/
typedef struct ss7_gen_config {
	char p_name[10][20];
	char p_value[10][40];
} sng_gen_t;

typedef struct ss7_mtp_config {
	char name[10];
	char p_name[10][20];
	char p_value[10][40];
} mtp_link_t;

typedef struct ss7_isup_config {
	char name[10];
	sng_gen_t sng_gen;
	mtp_link_t mtp1_link[10];
	mtp_link_t mtp2_link[10];
} sng_isup_t;

/********************************************************************************/
/*                                                                              */
/*                            FUNCTION DECLARATION                              */
/*                                                                              */
/********************************************************************************/
void test_mtp2_links(void);
void print_status(void);
void print_frames(void);
void verify_receive_data(chan_data_t *chan, void *vdata, uint32_t datalen);

#endif							/* __RUN_MTP2_H__ */
